﻿using System;
using System.IO;
using System.Linq;
using System.Web.Mvc;
using System.Web.Routing;
using AutoDbCreate.WebHost.Models;
using AutoDbCreate.WebHost.Services;
using Castle.ActiveRecord;
using Castle.Facilities.ActiveRecordIntegration;
using Castle.Facilities.Logging;
using Castle.MicroKernel;
using Castle.MicroKernel.Registration;
using Spark.Web.Mvc;

namespace AutoDbCreate.WebHost
{
    public class Application
    {
        public void RegisterViewEngines(ViewEngineCollection engines)
        {
            if (engines == null) throw new ArgumentNullException("engines");

            SparkEngineStarter.RegisterViewEngine(engines);
        }

        public void RegisterRoutes(RouteCollection routes)
        {
            if (routes == null) throw new ArgumentNullException("routes");

            // default route
            routes.Add(new Route(
                           "{controller}/{action}/{id}",
                           new RouteValueDictionary(new
                                                        {
                                                            controller = "home", 
                                                            action = "index", 
                                                            id = ""
                                                        }),
                           new MvcRouteHandler()));
        }

        public void InitializeStorage(IKernel kernel)
        {
            if (ShouldInitializeStorage)
            {
                string dataDirectory = Convert.ToString(AppDomain.CurrentDomain.GetData("DataDirectory"));
                string mdfPath = Path.Combine(dataDirectory, "admin.mdf");
                if (!File.Exists(mdfPath))
                {
                    var localDbManager = kernel.Resolve<ILocalDbManager>();
                    localDbManager.CreateDatabase(mdfPath);
                    localDbManager.CreateSchema(mdfPath, ARSchema.CUSTOMER);
                    localDbManager.CreateSchema(mdfPath, ARSchema.LOGISTIC);
                }
                ActiveRecordStarter.UpdateSchema();

            }

            if (ShouldInitializeData)
            {
                var dataInitializationService = kernel.Resolve<IDataInitializationService>();
                dataInitializationService.InitializeData(); 
                kernel.ReleaseComponent(dataInitializationService);
            }
        }

        public bool ShouldInitializeData { get; set; }
        public bool ShouldInitializeStorage { get; set; }


        public void RegisterComponents(IKernel kernel)
        {
            kernel.Register(
                Component
                    .For<IControllerFactory>()
                    .ImplementedBy<ControllerFactory>(),
               
               // Register generic repos
               Component.For(typeof(IRepository<>))
                    .ImplementedBy(typeof(Repository<>)),

                // Register all controllers in assembly. 
                AllTypes
                    .Of<IController>()
                    .FromAssembly(typeof (Application).Assembly)
                    .Configure(
                    reg =>
                    reg.Named(reg.Implementation.Name.ToLowerInvariant()).LifeStyle.Transient),
                
                // Register all services that implement marker interface in current assembly
                AllTypes.
                    Of<IService>()
                    .FromAssembly(typeof (Application).Assembly)
                    .WithService.FromInterface());

                
        }

        public void RegisterFacilities(IKernel kernel)
        {
            kernel.AddFacility("logging", new LoggingFacility(LoggerImplementation.Trace));
            kernel.AddFacility("activerecord", new ActiveRecordFacility());

        }
    }
}